/* ****************************************************	*
 *		## localStuff.h  ##			*
 *		## ----------- ##			*
 *							*
 * This file was generated automatically by 'siteconf'.	*
 * Don't edit this file, edit 'siteconf' instead.	*
 *							*
 * ****************************************************	*/
#ifdef host_is_BIG_ENDIAN
#undef host_is_BIG_ENDIAN
#endif
#ifdef host_is_LITTLE_ENDIAN
#undef host_is_LITTLE_ENDIAN
#endif
#define SIZEOF_CHAR 1
#define SIZEOF_SHORT_INT 2
#define SIZEOF_INT 4
#define HAVE_U_INTXX_T 1
#define HAVE_UINTXX_T 2
#define LOCAL_HAVE_inet_aton
#define LOCAL_HAVE_inet_pton
#define host_is_BIG_ENDIAN
